@@ -43,4 +43,12 @@ void asn_set_del(void *asn_set_of_x, int number, int _do_free);
  */
 void asn_set_empty(void *asn_set_of_x);
 
+/*
+ * Cope with different conversions requirements to/from void in C and C++.
+ * This is mostly useful for support library.
+ */
+typedef A_SET_OF(void) asn_anonymous_set_;
+#define _A_SET_FROM_VOID(ptr)		((asn_anonymous_set_ *)(ptr))
+#define _A_CSET_FROM_VOID(ptr)		((const asn_anonymous_set_ *)(ptr))
+
 #endif	/* ASN_SET_OF_H */
